package com.genuitec.pulse.client.delivery.example.launch.library.executable;

import java.io.File;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.equinox.p2.engine.IProfile;
import org.eclipse.equinox.p2.engine.IProfileRegistry;
import org.eclipse.equinox.p2.metadata.IInstallableUnit;
import org.eclipse.equinox.p2.query.IQueryResult;
import org.eclipse.equinox.p2.query.QueryUtil;

import com.genuitec.pulse.client.delivery.example.launch.library.executable.model.LaunchExecutableModel;
import com.genuitec.pulse.client.deliverypackage.runtime.internal.P2Services;
import com.google.eclipse.mechanic.Evaluator;
import com.google.eclipse.mechanic.IResourceTaskReference;
import com.google.eclipse.mechanic.RepairAction;
import com.google.eclipse.mechanic.Task;

@SuppressWarnings("restriction")
public class LaunchExecutableTask implements Task, Evaluator, RepairAction {

	private IResourceTaskReference reference;

	public LaunchExecutableTask(IResourceTaskReference reference) {
		this.reference = reference;
	}

	@Override
	public String getId() {
		return "com.genuitec.pulse.client.delivery.example.launch.library.executable";
	}

	@Override
	public String getTitle() {
		return "Launch Library Executable";
	}

	@Override
	public String getDescription() {
		return "Launch Library Executable";
	}

	@Override
	public Evaluator getEvaluator() {
		return this;
	}

	@Override
	public RepairAction getRepairAction() {
		return null;
	}

	@Override
	public boolean evaluate() {
		if (getReference() == null) {
			Logger.getLogger(getClass().getName()).log(Level.INFO, "Invalid task");
			return true;
		}
		Logger.getLogger(getClass().getName()).log(Level.INFO, "Evaluating task");
		boolean installDirectoryExist = false;
		LaunchExecutableModel launchExecutable = null;
		try {
			installDirectoryExist = getInstallDirectory() != null && getInstallDirectory().exists();
			launchExecutable = unmarshalFromFile(LaunchExecutableModel.class, getReference().asFile(), true);
		} catch (Exception e) {
			Logger.getLogger(getClass().getName()).log(Level.SEVERE, "Install directory does not exist", e);
		}
		boolean valid = launchExecutable != null && installDirectoryExist;

		if (valid) {
			Logger.getLogger(getClass().getName()).log(Level.INFO, "Valid task");
			valid = launchExecutable(launchExecutable);
			Logger.getLogger(getClass().getName()).log(Level.INFO, "Preference changed");
		} else {
			Logger.getLogger(getClass().getName()).log(Level.INFO, "Invalid task");
		}

		return true;
	}

	private boolean launchExecutable(LaunchExecutableModel launchExecutable) {
		boolean executed = false;
		try {
			File exeFile = getExeFile(launchExecutable);
			
			if (exeFile != null && exeFile.exists()) {
				List<String> commands = new ArrayList<String>();
				commands.add(exeFile.getAbsolutePath());
				if (launchExecutable.getExecutableArguments() != null) {
					for (String arg : launchExecutable.getExecutableArguments()) {
						commands.add(arg);
					}
				}
				
				ProcessBuilder proc = new ProcessBuilder();
				proc.command(commands.toArray(new String[commands.size()]));
				executed = true;
			}
		} catch (Exception e) {
			Logger.getLogger(getClass().getName()).log(Level.WARNING, "Can not execute file", e);
		}
		return executed;
	}

	private File getExeFile(LaunchExecutableModel launchExecutable) throws IOException, IllegalArgumentException, InterruptedException {
		File result = null;
		IPath path = new Path(getInstallDirectory().getAbsolutePath());

		path = path.append("/binary/" + getIUId(launchExecutable));

		if (Platform.OS_WIN32.equals(Platform.getOS())) {
			result = findFileInDirectory(path.toFile(), launchExecutable.getWindowsExecutableName());
		} else if (Platform.OS_LINUX.equals(Platform.getOS())) {
			result = findFileInDirectory(path.toFile(), launchExecutable.getLinuxExecutableName());
			ProcessBuilder proc = new ProcessBuilder();
			proc.command("/bin/chmod", "755", result.getAbsolutePath());
			proc.start().waitFor();
		} else if (Platform.OS_MACOSX.equals(Platform.getOS())) {
			result = findFileInDirectory(path.toFile(), launchExecutable.getOsxExecutableName());
			ProcessBuilder proc = new ProcessBuilder();
			proc.command("/bin/chmod", "755", result.getAbsolutePath());
			proc.start().waitFor();
		}
		
		if (result != null) {
			Logger.getLogger(getClass().getName()).log(Level.INFO, MessageFormat.format("Executable location: {0}", result.getAbsolutePath()));
		} else {
			Logger.getLogger(getClass().getName()).log(Level.INFO, "Executable file was not found");
		}

		return result;
	}

	private String getIUId(LaunchExecutableModel launchExecutable) throws IllegalArgumentException {
		Logger.getLogger(getClass().getName()).log(Level.INFO, "Looking for " + launchExecutable.getLibraryName() + " on running profile");
		IProfile profile = getCurrentProfile();

		if (profile == null) {
			Logger.getLogger(getClass().getName()).log(Level.INFO, "Can not find running profile");
		}

		IQueryResult<IInstallableUnit> query = profile.query(QueryUtil.createIUPropertyQuery("org.eclipse.equinox.p2.name", launchExecutable.getLibraryName()), new NullProgressMonitor());

		if (query == null || query.isEmpty()) {
			Logger.getLogger(getClass().getName()).log(Level.INFO, "Can not find " + launchExecutable.getLibraryName() + " on running profile");
			throw new IllegalArgumentException();
		}
		return query.toSet().iterator().next().getId();

	}

	public static IProfile getCurrentProfile() {
		IProfileRegistry profileRegistry = P2Services.getProfileRegistry();
		IProfile profile = profileRegistry.getProfile(IProfileRegistry.SELF);
		return profile;
	}

	public File findFileInDirectory(File root, String findFile) {
		if (!root.isDirectory())
			return null;
		File[] list = root.listFiles();
		if (list == null)
			return null;

		for (File f : list) {
			if (f.isDirectory()) {
				return findFileInDirectory(f, findFile);
			} else if (f.isFile() && findFile.equals(f.getName())) {
				return f;
			}
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	public <E> E unmarshalFromFile(Class<E> root, File file, boolean strict) throws JAXBException {
		JAXBContext context = JAXBContext.newInstance(root);
		Unmarshaller unmarshaller = context.createUnmarshaller();
		return (E) unmarshaller.unmarshal(file);
	}

	private File getInstallDirectory() throws IOException {
		File installDirectory = new File(FileLocator.toFileURL(Platform.getInstallLocation().getURL()).getFile());
		return installDirectory;
	}

	@Override
	public int getRepairCost() {
		return 0;
	}

	@Override
	public IStatus repair(IProgressMonitor monitor) {
		//do nothing
		return Status.OK_STATUS;
	}

	@Override
	public IResourceTaskReference getReference() {
		return this.reference;
	}

}
